function [funcVal, w, lambda, xi] = nnpull_sparseTrain(K, ps, options)
[p, n] = size(ps);

% build the following LP
% min \sum_i c_i + C*n*\gamma + D\sum_i \xi_i
% s.t. |f^j(x_i, w) - p_i^j| <= \gamma max_j |p_i^j| + \xi_i 
%      w_i <= c_i
%      w_i >= -c_i
%      0 <= \gamma <= 1
%      \xi_i \ge 0
%
% where f^j(x_i, w) = \sum_k w_k p_k^j K(x_i, x_k) and K is the kernel function.

% number of variables 3n + 1, c_i: n, w_i : n, \xi_i : n, \gamma : 1, 
% order: w_i, c_i, \xi_i, \gamma
% number of constraints 2*n*p + 2*n (except for lower and upper bound)

if isfield(options, 'xi') && options.xi
    options.xi = true;
end;

if options.xi
    nVar = 3*n + 1;
else
    nVar = 2*n + 1;
end;
nConstraint = 2*n*p + 2*n;

if options.xi
    problem.f = [zeros(n, 1); ones(n, 1); repmat(options.D, n, 1); options.C * n];
else
    problem.f = [zeros(n, 1); ones(n, 1); options.C * n];
end;

% Aineq * x \le bineq
problem.Aineq = zeros(nConstraint, nVar);
problem.bineq = zeros(nConstraint, 1);

% max_j |p_i^j|
psmaxs = max(abs(ps), [], 1);

fprintf(1, 'n = %d, p = %d\n', n, p);
fprintf(1, 'constraints |f^j(x_i, w) - p_i^j| <= gamma max_j |p_i^j| + xi_i...\n');
% constraint |f^j(x_i, w) - p_i^j| <= \gamma max_j |p_i^j| + \xi_i 
for i = 1:n
    for j = 1:p
        index = (i - 1) * p + j;
        coeff = (ps(j, :) .* K(i, :))';
        problem.Aineq(index, 1:n) = coeff;
        problem.Aineq(index, end) = -psmaxs(i); % lambda
        problem.bineq(index) = ps(j, i);
        
        problem.Aineq(index + n*p, 1:n) = -coeff;
        problem.Aineq(index + n*p, end) = -psmaxs(i); % lambda
        problem.bineq(index + n*p) = -ps(j, i);
        
        if options.xi
            problem.Aineq(index, 2*n+i) = -1; % xi
            problem.Aineq(index + n*p, 2*n+i) = -1; % xi
        end;
    end;
end;

fprintf(1, 'constraints |w_i| <= c_i...\n');
% constraints |w_i| <= c_i 
for i = 1:n
    problem.Aineq(i + 2*n*p, i) = 1;
    problem.Aineq(i + 2*n*p, n + i) = -1;
    
    problem.Aineq(i + 2*n*p + n, i) = -1;
    problem.Aineq(i + 2*n*p + n, n + i) = -1;
end;

problem.Aineq = sparse(problem.Aineq);
problem.bineq = sparse(problem.bineq);

% solve it!!
largeNum = 1e9;
if options.xi
    problem.lb = [repmat(-largeNum, n, 1); zeros(n, 1); zeros(n, 1); 0];
    problem.ub = [repmat(largeNum, n, 1);  repmat(largeNum, n, 1); repmat(largeNum, n, 1); 1]; 
else
    problem.lb = [repmat(-largeNum, n, 1); zeros(n, 1); 0];
    problem.ub = [repmat(largeNum, n, 1);  repmat(largeNum, n, 1); 1]; 
end;
problem.solver = 'linprog';
problem.options = optimset('LargeScale', 'on', 'Simplex', 'off', 'MaxIter', 100000);
% initial condition
if options.xi
    problem.x0 = [zeros(n, 1); zeros(n, 1); zeros(n, 1); 1];
else
    problem.x0 = [zeros(n, 1); zeros(n, 1); 1];
end;

fprintf(1, 'run linear programming...\n');
[solution, funcVal, flag] = linprog(problem);
if flag ~= 1
    fprintf(1, 'solution may be wrong!\n');
end;

w = solution(1:n);
if options.xi
    xi = solution(2*n+1:3*n);
else
    xi = [];
end;
lambda = solution(end);